create database project_management;

use project_management;


create table groups
(
	id int default null auto_increment, 
	name char(50) not null unique,
	description char(200) default null,
	primary key (id)
)engine=innodb;

create table permitions 
(
	id int default null auto_increment, 
	primary key (id),
	controller char(50) not null,
	action char(50) not null,
	group_id char(20) not null,
	readonly TINYINT(1) default false
)engine=innodb;

create table users 
(
	id int default null auto_increment, 
	primary key (id),
	usr char(10) not null unique,
	pass char(10) not null,
	fname char(15) not null,
	lname char(15) not null,
	group_id int,
	index ind_group_id (group_id),
	foreign key (group_id) references groups(id) on delete no action,
	team_id int default null,
	phone char(15),
	email char(25)
)engine=innodb;

create table teams 
(
	id int default null auto_increment, 
	primary key (id),
	name char(20) not null unique,
	leader_id int,
	index ind_leader_id (leader_id),
	foreign key (leader_id) references users (id) on delete no action,
	description char(100) default ""
)engine=innodb;

create table projects 
(
	id int default null auto_increment, 
	primary key (id),
	name char(20) not null unique,
	description char(100) default "",
	team_id int,
	index ind_team_id (team_id),
	foreign key (team_id) references teams (id) on delete set null,
	startTime timestamp,
	endTime timestamp,
	status TINYINT(1) default false
)engine=innodb;



create table documents 
(
	id int default null auto_increment, 
	primary key (id),
	file_name char(100) not null,
	version int(10) not null,
	author_id int,
	index ind_author_id (author_id),
	foreign key (author_id) references users (id) on delete set null,
	obj_type char(20) not null,
	obj_id int not null
)engine=innodb;


create table tasks
(
	id int default null auto_increment, 
	primary key (id),
	name char(50) not null unique,
	description char(100) default "",
	project_id int,
	index ind_project_id (project_id),
	foreign key (project_id) references projects (id) on delete set null,
	author_id int,
	index ind_author_id (author_id),
	foreign key (author_id) references users (id) on delete set null,
	owner_id int,
	index ind_owner_id (owner_id),
	foreign key (owner_id) references users (id) on delete set null,
	startTime timestamp,
	parent_id int,
	index ind_parent_id (parent_id),
	foreign key (parent_id) references tasks (id) on delete cascade,
	endTime timestamp,
	priority int(1) default 1,
	status TINYINT(1) default false
)engine=innodb;

create table comments
(
    id int default null auto_increment, 
	primary key (id),
	time timestamp,
	author_id int,
	index ind_author_id (author_id),
	foreign key (author_id) references users (id) on delete set null,
	description char(100) default "",
	obj_type char(20) not null,
	obj_id int not null
)engine=innodb;

INSERT INTO groups (id,name,description) VALUES (NULL,'admin','The administrator of application. Full permitions. The default admin is admin/admin');
INSERT INTO groups (id,name,description) VALUES (NULL,'pm','Project Manager');
INSERT INTO groups (id,name,description) VALUES (NULL,'tl','Team Leader');
INSERT INTO groups (id,name,description) VALUES (NULL,'architect','Architect');
INSERT INTO groups (id,name,description) VALUES (NULL,'dev','Developer');



INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('account', '*', '*', false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('document', '*', '*',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('main', '*', '*', false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', '*', '*', true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'show', 'dev',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'show', 'architect',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'delete', 'dev',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'delete', 'architect',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'list', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'show', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('project', 'delete', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('task', '*', '*',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', 'delete', 'architect',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', 'delete', 'dev',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', '*', '*',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', 'list', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', 'show', 'dev',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('team', 'show', 'architect',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', 'show', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', 'list', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', 'save', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', 'update_group', 'admin',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', 'delete', 'admin', false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('user', '*', '*',	true);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('comment', '*', '*',	false);
INSERT INTO permitions (controller, action, group_id, readonly) VALUES ('search', '*', '*',	fase);

INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'admin','admin','Administrator','Administrator',1,NULL);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'minko','minko','Minko','G',5,1);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'rosko','rosko','Rostislav','G',4,1);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'pesho','pesho','Petar','P',3,1);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'nadq','nadq','Nadejda','Ch',3,2);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'niki','niki','Nikolaj','P',4,2);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'jeni','jeni','Jeni','D',2,NULL);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'desi','desi','Desislava','P',2,NULL);
INSERT INTO users (id,usr,pass,fname,lname,group_id,team_id) VALUES (NULL,'vesy','vesy','Az','az',5,1);


INSERT INTO teams (id, name, leader_id, description) VALUES (NULL,'bas',4,'developing bas components');
INSERT INTO teams (id, name, leader_id, description) VALUES (NULL,'app',5,'application');

INSERT INTO projects (id,name,description,team_id, status) VALUES (NULL,'calendar','calendar description',1, 1);
INSERT INTO projects (id,name,description,team_id, status) VALUES (NULL,'account','description of account project',1, 1);

INSERT INTO projects (id,name,description,team_id, status, startTime) VALUES (NULL,'fitness','description of fitness project',2, 1,  '2006-11-04 19:20:07');

INSERT INTO tasks (id,name,project_id, author_id, owner_id) VALUES (NULL,'weekly view',1,4,4);
INSERT INTO tasks (id,name,project_id, author_id, owner_id) VALUES (NULL,'now task',1,4,9);
INSERT INTO tasks (id,name,project_id, author_id, owner_id, parent_id, status) VALUES (NULL,'todo',1,4,9,2,1);
INSERT INTO tasks (id,name,project_id, author_id, owner_id, parent_id, status) VALUES (NULL,'todo second',1,4,9,2,1);
INSERT INTO tasks (id,name,project_id, author_id, owner_id, parent_id, status) VALUES (NULL,'todo 3',1,4,9,2,1);
INSERT INTO tasks (id,name,project_id, author_id, owner_id) VALUES (NULL,'now aha',1,4,9);
